import argparse
import os
import shutil

from .util import *
from .gen_doc import generate_web_docs
from .zood import *
from .extensions import update_PYPI_package, update_vsce_package


def main():
    parser = argparse.ArgumentParser(
        description="zood: web page documentation & comment generation documentation"
    )
    parser.add_argument("cmd", type=str, nargs="*", help="initialize docs template")
    parser.add_argument("-g", "--generate", action="store_true", help="generate html doc")
    parser.add_argument(
        "-s",
        "--save",
        action="store_true",
        help="save _config.yml and use in every environment",
    )
    args = parser.parse_args()

    config = get_zood_config()  # 获取配置信息
    md_dir_name = config["markdown_folder"]

    local_config_path = os.path.join(md_dir_name, "_config.yml")
    global_config_path = os.path.join(os.path.dirname(__file__), "config", "_config.yml")

    if args.generate:
        if not os.path.exists(md_dir_name):
            # 递归的在上级目录寻找
            return
        generate_web_docs(md_dir_name)
        return

    if args.save:
        if os.path.exists(local_config_path):
            shutil.copy(local_config_path, global_config_path)
            print_info("已更新全局配置文件 _config.yml", color="green")
        else:
            print("未找到", local_config_path)

        return

    if len(args.cmd) == 0 or args.cmd[0] == "help":
        show_help_info()
        return

    if args.cmd[0] == "init":
        init_zood(md_dir_name)

    elif args.cmd[0] == "new":
        if len(args.cmd) == 2:
            dir_name = "."
            file_name = args.cmd[1]
        elif len(args.cmd) == 3:
            dir_name = args.cmd[1]
            file_name = args.cmd[2]
            if dir_name == md_dir_name:
                print_info(f"您不能创建一个和 {md_dir_name} 同名的子文件夹")
                return
        else:
            print_info(f"创建新文件的命令为 zood new [目录] [文件名]")
            return

        if not os.path.exists(md_dir_name):
            init_zood(md_dir_name)

        create_new_file(md_dir_name, dir_name, file_name)

    elif args.cmd[0] == "clean":
        shutil.rmtree("docs")
        print_info(f"已删除 docs")

    elif args.cmd[0] == "config":
        shutil.copy(global_config_path, local_config_path)
        print_info(f"生成配置文件 {local_config_path}", color="green")

    elif args.cmd[0] == "update":
        current_dir = os.getcwd()
        dir_yml_path = os.path.join(current_dir, md_dir_name, "dir.yml")
        dir_yml = read_configfile(dir_yml_path)
        sort(dir_yml)
        for dir_name, files in dir_yml.items():
            for i in range(len(files)):
                file_name = list(files[i].keys())[0]
                file_path = os.path.join(md_dir_name, dir_name, file_name + ".md")
                if not os.path.exists(file_path):
                    print("未找到", file_name)
                files[i][file_name] = i + 1

        writeConfigFile(dir_yml, dir_yml_path)
        print("已更新排序")

    elif args.cmd[0] == "poetry":
        if len(args.cmd) > 2:
            print_info("peotry 接收0/1个参数")
            show_help_info()
            return

        choice = None
        if len(args.cmd) == 2:
            choice = args.cmd[1]
            if choice not in ("sub", "main"):
                print_info("choice 选项为 (sub, main)")
                show_help_info()
                return
        update_PYPI_package(choice)

    elif args.cmd[0] == "vsce":
        if len(args.cmd) > 2:
            print_info("vsce 接收0/1个参数")
            show_help_info()
            return

        choice = None
        if len(args.cmd) == 2:
            choice = args.cmd[1]
            if choice not in ("sub", "main"):
                print_info("choice 选项为 (sub, main)")
                show_help_info()
                return
        update_vsce_package(choice=choice)

    elif args.cmd[0] == 'log':
        # 输出错误日志信息
        with open(os.path.join(os.path.dirname(__file__),'config', 'error.log'), 'r', encoding='utf-8') as f:
            print(f.read())

    else:
        print_info(f"未找到指令 zood {args.cmd[0]}")
        show_help_info()


def show_help_info():
    print("zood使用方法见 https://luzhixing12345.github.io/zood/\n")
    print("{:<20}初始化仓库".format("  zood init"))
    print("{:<20}创建A目录下的B文件".format("  zood new A B"))
    print("{:<20}创建根目录下的A文件".format("  zood new A"))
    print("{:<20}更新dir.yml顺序".format("  zood update"))
    print("{:<20}生成docs/目录".format("  zood -g"))
    print("{:<20}删除docs/目录".format("  zood clean"))
    print("{:<20}获取配置文件".format("  zood config"))
    print("{:<20}输出错误信息".format("  zood log"))
    print("{:<20}更新配置文件".format("  zood -s"))

    print("\n其他:")
    print("{:<25}更新PYPI库版本\n".format("  zood poetry <choice>"))
    indent = 12
    print(" " * indent, "choice = None(default) 发布版本更新")
    print(" " * indent, "choice = sub           次版本更新")
    print(" " * indent, "choice = main          主版本更新")
    print("")
    print("{:<25}更新Vscode扩展版本\n".format("  zood vsce <choice>"))
    indent = 12
    print(" " * indent, "choice = None(default) 发布版本更新")
    print(" " * indent, "choice = sub           次版本更新")
    print(" " * indent, "choice = main          主版本更新")
    print("")



import shutil
import os
from .util import *

def init_zood(md_dir_name):
    # 初始化 zood 文件

    if os.path.exists(md_dir_name):
        print_info(f"{md_dir_name} 已存在,请删除文件夹后重试")
        return
    else:
        os.mkdir(md_dir_name)

        readme = ""
        if os.path.exists("README.md"):
            with open("README.md", "r", encoding="utf-8") as f:
                readme = f.read()

        init_dir_yml(md_dir_name)

        with open(os.path.join(md_dir_name, "README.md"), "w", encoding="utf-8") as f:
            f.write(readme)

        print_info(f"已初始化 [{md_dir_name}]", "green")


def create_new_file(md_dir_name, dir_name, file_name):
    file_path = os.path.join(md_dir_name, dir_name, file_name + ".md")

    if os.path.exists(file_path):
        print_info(f"{file_path} 已存在")
        return

    if not os.path.exists(os.path.join(md_dir_name, dir_name)):
        os.makedirs(os.path.join(md_dir_name, dir_name))

    update_dir_yml(file_name, dir_name, md_dir_name)

    with open(file_path, "w", encoding="utf-8") as f:
        basic_info = f"\n# {file_name}\n"
        f.write(basic_info)

    print_info(f"创建文件 {file_path}", color="green")


def init_dir_yml(md_dir_name):
    # 生成目录记录
    dir_yml = {".": [{"README": 1}]}
    writeConfigFile(dir_yml, os.path.join(md_dir_name, "dir.yml"))


def update_dir_yml(file_name, dir_name, md_dir_name):
    # 更新当前路径下的 dir.yml

    current_dir = os.getcwd()
    dir_yml_path = os.path.join(current_dir, md_dir_name, "dir.yml")
    dir_yml = read_configfile(dir_yml_path)

    sort(dir_yml)

    if dir_name in dir_yml.keys():
        number = list(dir_yml[dir_name][-1].values())[0] + 1
        dir_yml[dir_name].append({file_name: number})
    else:
        dir_yml[dir_name] = []
        dir_yml[dir_name].append({file_name: 1})

    writeConfigFile(dir_yml, dir_yml_path)




def parse_config(config, markdown_htmls):
    html_dir_name = config["html_folder"]
    html_tempate_path = os.path.join(os.path.dirname(__file__), "config", "template.html")
    css_template_path = os.path.join(os.path.dirname(__file__), "config", "index.css")

    with open(html_tempate_path, "r", encoding="utf-8") as f:
        basic_html_template = f.read()

    # js 部分
    js_scope = zood_js_options(config, markdown_htmls)
    basic_html_template = basic_html_template.replace("js-scope", js_scope)

    # css 部分
    # prism_src = "../../../css/prism.css"
    index_src = "../../../css/index.css"

    css_scope = ""
    # if config["options"]["enable_highlight"]:
    #     css_scope += f"<link rel='stylesheet' href={prism_src} />"
    css_scope += f"<link rel='stylesheet' href={index_src} />"
    basic_html_template = basic_html_template.replace("css-scope", css_scope)

    with open(css_template_path, "r", encoding="utf-8") as f:
        basic_css_template = f.read()
    favicon_url: str = config["favicon"]
    if favicon_url[:4] == "http":
        basic_html_template = basic_html_template.replace("<%favicon%>", favicon_url)

    else:
        img_name = favicon_url.split(os.sep)[-1]
        shutil.copy(favicon_url, f"./{html_dir_name}/img/" + img_name)
        basic_html_template = basic_html_template.replace("<%favicon%>", "../../../img/" + img_name)

    title = config["title"]
    basic_html_template = basic_html_template.replace("<%title%>", title)

    custom_options = get_custom_options(config, ["color", "font", "position"])

    for k, v in custom_options:
        basic_css_template = basic_css_template.replace(f'"<%{k}%>"', v)

    basic_html_template = basic_html_template.replace("css-scope", css_scope)

    with open(f"./{html_dir_name}/css/index.css", "w", encoding="utf-8") as f:
        f.write(basic_css_template)

    return basic_html_template


def get_custom_options(config, keys):
    custom_options = []
    for k in keys:
        custom_options += list(config[k].items())
    return custom_options


def zood_js_options(config, markdown_htmls):
    js_scope = ""
    html_dir_name = config["html_folder"]
    md_dir_name = config["markdown_folder"]

    if config["options"]["enable_next_front"]:
        js_code = insertJScode("enable_next_front", html_dir_name)
        js_code += f"<script>addLink(<%front_url%>,<%next_url%>,<%control%>)</script>"
        js_scope += js_code

    if config["options"]["enable_change_mode"]:
        js_code = insertJScode("enable_change_mode", html_dir_name)
        js_code += (
            f'<script>addChangeModeButton("../../../img/sun.png","../../../img/moon.png")</script>'
        )
        js_scope += js_code

    if config["options"]["enable_copy_code"]:
        js_code = insertJScode("enable_copy_code", html_dir_name)
        js_code += f'<script>addCodeCopy("../../../img/before_copy.png","../../../img/after_copy.png")</script>'
        js_scope += js_code

    if config["options"]["enable_navigator"]:
        js_code = insertJScode("enable_navigator", html_dir_name)
        js_scope += js_code

    # if config["options"]["enable_highlight"]:
    #     # 复制prismjs
    #     shutil.copy(
    #         os.path.join(os.path.dirname(__file__), "config", "js", "prismjs", "prism.css"),
    #         f"{html_dir_name}/css",
    #     )
    #     shutil.copy(
    #         os.path.join(os.path.dirname(__file__), "config", "js", "prismjs", "prism.js"),
    #         f"{html_dir_name}/js",
    #     )
    #     src = "../../../js/prism.js"
    #     highlight = f'<script type="text/javascript" src="{src}"></script>'
    #     js_scope += highlight

    if config["options"]["enable_picture_title"]:
        js_code = insertJScode("enable_picture_title", html_dir_name)
        js_scope += js_code

    if config["options"]["enable_picture_preview"]:
        js_code = insertJScode("enable_picture_preview", html_dir_name)
        js_scope += js_code

    if config["options"]["enable_search"]["enable"]:
        js_code = insertJScode("enable_search", html_dir_name)
        all_api_text = getAllAPIText(
            markdown_htmls,
            config["options"]["enable_search"]["search_scope"],
            md_dir_name,
        )
        js_code += f'<script>addSearchBar({all_api_text},"../../../img/search.svg","../../../img/enter.svg","Ctrl+K")</script>'
        js_scope += js_code

    if config["options"]["enable_mermaid"]:
        js_code = "<script type=\"module\">\
const codeBlocks = document.querySelectorAll('.language-mermaid');\
codeBlocks.forEach(codeBlock => {\
    codeBlock.classList.remove('language-mermaid');\
    codeBlock.classList.add('mermaid');\
});\
import mermaid from 'https://cdn.jsdelivr.net/npm/mermaid@10/dist/mermaid.esm.min.mjs';\
mermaid.initialize({ startOnLoad: true });\
</script>"
        js_scope += js_code

    if config["options"]["enable_latex"]:
        js_code = """
        <script>
            MathJax = {
            tex: {
                inlineMath: [['$', '$'], ['\\(', '\\)']]
            }
            };
            </script>
        <script id="MathJax-script" async
        src="https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-chtml.js">
        </script>
        """
        js_scope += js_code

    js_scope += insertJScode("global_js_configuration", html_dir_name)

    return js_scope


def insertJScode(file_name: str, html_dir_name):
    if file_name.startswith("enable_"):
        # 跳过enable_
        file_name = file_name[7:]
    js_file_name = os.path.join(os.path.dirname(__file__), "config", "js", f"{file_name}.js")
    if os.path.exists(js_file_name):
        shutil.copy(
            js_file_name,
            f"{html_dir_name}/js",
        )
    src = f"../../../js/{file_name}.js"
    js_code = f'<script type="text/javascript" src="{src}"></script>'
    return js_code


